package org.xbib.datastructures.api;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * A {@code Multimap} that can hold duplicate key-value pairs and that maintains the insertion
 * ordering of values for a given key. See the {@link Multimap} documentation for information common
 * to all multimaps.
 *
 * <p>The {@link #get}, {@link #removeAll}, and {@link #replaceValues} methods each return a {@link
 * List} of values. Though the method signature doesn't say so explicitly, the map returned by
 * {@link #asMap} has {@code List} values.
 */
public interface ListMultimap<K extends Object, V extends Object>
        extends Multimap<K, V> {
    /**
     * {@inheritDoc}
     *
     * <p>Because the values for a given key may have duplicates and follow the insertion ordering,
     * this method returns a {@link List}, instead of the {@link Collection} specified in
     * the {@link Multimap} interface.
     */
    @Override
    List<V> get(K key);

    /**
     * {@inheritDoc}
     *
     * <p>Because the values for a given key may have duplicates and follow the insertion ordering,
     * this method returns a {@link List}, instead of the {@link Collection} specified in
     * the {@link Multimap} interface.
     */
    @Override
    List<V> removeAll(Object key);

    /**
     * {@inheritDoc}
     *
     * <p>Because the values for a given key may have duplicates and follow the insertion ordering,
     * this method returns a {@link List}, instead of the {@link Collection} specified in
     * the {@link Multimap} interface.
     */
    @Override
    List<V> replaceValues(K key, Iterable<? extends V> values);

    /**
     * {@inheritDoc}
     *
     * <p><b>Note:</b> The returned map's values are guaranteed to be of type {@link List}.
     */
    @Override
    Map<K, Collection<V>> asMap();

    /**
     * Compares the specified object to this multimap for equality.
     *
     * <p>Two {@code ListMultimap} instances are equal if, for each key, they contain the same values
     * in the same order. If the value orderings disagree, the multimaps will not be considered equal.
     *
     * <p>An empty {@code ListMultimap} is equal to any other empty {@code Multimap}, including an
     * empty {@code SetMultimap}.
     */
    @Override
    boolean equals(Object obj);
}
